#include <iostream>
#include <cstdarg>
#include <vector>
#include <cstdio>

using namespace std;

#include "fraction"

static int verbose;

template<class sourcetype>
class algofinder
{
private:
    //typedef fraction<sourcetype> tmptype;
    typedef sourcetype tmptype;

    struct elem
    {
        tmptype integer;        /* Number this is multiplied by      */
        unsigned exponent;  /* 0=constant, 1=x, 2=xx, 3=xxx, ... */
    public:
        elem() : integer(0), exponent(0) { }
        elem(int add, unsigned exp) : integer(add), exponent(exp) { }
    };
    
    bool NearEnough(const tmptype &got, const tmptype &wanted) const
    {
        if(got == wanted)
            return true;

        tmptype diff;
        diff = (got > wanted)
                ? (got-wanted)
                : (wanted-got);
        
        if(diff.value() < 0.1)return true;
        
        return false;
    }
    
    bool Test(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values,
        const vector<algofinder<sourcetype>::elem> &guess) const
    {
        bool wasok = true;
        
        /* Test the guess */

        for(unsigned a=0; a<keys.size(); ++a)
        {
            tmptype val = keys[a];
            tmptype result = 0;
                
            for(unsigned b=0; b<guess.size(); ++b)
            {
                tmptype thisval = 1;
                for(unsigned c=0; c<guess[b].exponent; ++c)thisval *= val;
                thisval *= guess[b].integer;
                result += thisval;
            }
            if(!NearEnough(result, (tmptype)values[a]))
            {
                /* Wrong */
                wasok = false;
                if(verbose > 1)
                {
                    cerr << "  Nope, ";
                    cerr.precision(10);
                    cerr << result.value();
                    cerr << " != " << values[a] << ": ";
                    goto Ouch;
                }
                return false;
            }
        }
        if(verbose > 1)
            cerr << "\nFound:  " << flush;

    Ouch:
        for(unsigned b=0; b<guess.size(); ++b)
        {
            if(guess[b].integer)
            {
                bool nega = guess[b].integer.negative();
                
                if(b)cout << ' ';
                
                sourcetype nom = guess[b].integer.nom();
                sourcetype denom=guess[b].integer.denom();
                
                if(nega ^ (nom < sourcetype(0))) { nom=-nom; denom=-denom; }
                
                if(nom == sourcetype(-1) && guess[b].exponent)
                {
                    cout << '-';
                    if(denom != sourcetype(1))cout << ' ';
                }
                else
                {
                    if(b && nom >= sourcetype(0))cout << '+';
                    if(abs(nom) > sourcetype(1) || !guess[b].exponent)cout << nom;
                }
                if(guess[b].exponent)
                {
                    cout << 'x';
                    if(guess[b].exponent == 2)
                        cout << '';
                    else if(guess[b].exponent > 1)
                        cout << '^' << guess[b].exponent;
                }
                if(denom != sourcetype(1))
                    cout << '/' << denom;
            }
        }
        cout << endl;
        
        if(wasok && verbose > 1) { cerr << endl; }
        
        return wasok;
    }

    tmptype calcdet(const vector<sourcetype> &mat, unsigned depf) const
    {
        tmptype sum;
        if(depf <= 1)
            sum = mat[0];
        else
        {
            sum = (tmptype)0;
            for(unsigned a=0; a<depf; ++a)
            {
                unsigned subdepf = depf-1;
                vector<sourcetype> tmp(subdepf * subdepf);
                unsigned pos = 0;
                unsigned sposbase = depf;
                for(unsigned b=0; b<subdepf; ++b)
                {
                    unsigned sposadd = 0;
                    for(unsigned c=0; c<subdepf; ++c)
                    {
                        if(c==a) if(++sposadd == depf)sposadd = 0;
                        tmp[pos++] = mat[sposbase + sposadd++];
                        if(sposadd==depf)sposadd=0;
                    }
                    sposbase += depf;
                }
                tmptype subdet = mat[a] * calcdet(tmp, subdepf);
                if(a & 1)
                    sum -= subdet;
                else
                    sum += subdet;
            }
        }
        return sum;
    }

public:
    void Guess(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values) const
    {
        if(verbose > 0)
        {
            cerr << "Analyzing {";
            for(unsigned a=0; a<keys.size(); ++a)
            {
                if(a) cerr << ',';
                cerr << keys[a];
            }
            cerr << "} => {";
            for(unsigned a=0; a<values.size(); ++a)
            {
                if(a) cerr << ',';
                cerr << values[a];
            }
            cerr << "}\n";
        }
        if(keys.size() != values.size())
        {
            cerr << "Error: keys.size() (" << keys.size() << ") != values.size() (" << values.size() << ")\n";
            return;
        }
        
        for(unsigned elemcount=1; elemcount<=keys.size(); ++elemcount)
        {
            vector<sourcetype> mat (elemcount * elemcount);
            
            for(unsigned a=0; a<elemcount; ++a)
                for(unsigned b=0; b<elemcount; ++b)
                {
                    sourcetype val = 1;
                    for(unsigned c=0; c<b; ++c)val *= keys[a];
                    mat[a*elemcount + b] = val;
                }
            
            tmptype det = calcdet(mat, elemcount);
            
            typedef algofinder<sourcetype>::elem elem;
            
            vector<elem> guess(elemcount);
            
            for(unsigned a = 0; a < elemcount; ++a)
            {
                vector<sourcetype> mat2 = mat;
                
                for(unsigned c = 0; c < elemcount; ++c)
                    mat2[c*elemcount+a] = values[c];
                
                guess[elemcount-1-a].integer  = calcdet(mat2, elemcount) / det;
                guess[elemcount-1-a].exponent = a;
            }
            
            if(Test(keys, values, guess))
                return;
        }
        if(verbose > 0)
            cerr << "Not found!\n";
    }

    template<typename itemtype>
    void Guess(
        const vector<itemtype> &keys,
        const vector<itemtype> &values) const
    {
        vector<sourcetype> fkeys(keys.size());
        vector<sourcetype> fvalues(keys.size());
        for(unsigned a=0; a<keys.size(); ++a)fkeys[a] = keys[a];
        for(unsigned a=0; a<values.size(); ++a)fvalues[a] = values[a];
        Guess(fkeys, fvalues);
    }

    vector<sourcetype> array(sourcetype a,sourcetype b,sourcetype c,sourcetype d) const
    {
        vector<sourcetype> result(4);
        result[0]=a;result[1]=b;result[2]=c;result[3]=d;
        return result;
    }

    void Test() const
    {
        /* Should output: x + 4 */     
        
        Guess( array(0,1,2,3),           /*    x*0 + y*1 = 4      x=(4*1-0*5) / (1-0)=4 */
               array(4,5,6,7) );         /*    x*1 + y*1 = 5      y=(5  -  4) / (1-0)=1 */
        
        /* Should output: 2x */

        Guess( array(10,11,12,13),       /*    x*10 + y*1 = 20    x=(20*11-22*10) / (11-10)=0 */
               array(20,22,24,26) );     /*    x*11 + y*1 = 22    y=(22   -20)    / (11-10)=2 */
        
        /* Should output: 3x + 1 */
        
        Guess( array(1,2,3,4),
               array(4,7,10,13) );
        
       /* Should output: xx - 1 */
        
        Guess( array(1,2,3,4),
               array(0,3,8,15) );

        /* Should output: xx - 2x +20 */
        
        Guess( array( 1, 2, 3, 4),    /*  x*1 + y*1 + z*1 = 19  x = 1*(2-3)+1(-9blah  */
               array(19,20,23,28) );  /*  x*4 + y*2 + z*1 = 20  it works anyway. :) */
                                      /*  x*9 + y*3 + z*1 = 23 */
        Guess( array( 1, 2, 3, 5),
               array(11,27,23,29) );
    }
};


#include <argh.hh>

int main(int argc, const char *const *argv)
{
    ParamHandler Argh;
    Argh.AddLong("help",    'h').SetBool().SetDesc("This help.");
    Argh.AddLong("version", 'V').SetBool().SetDesc("Displays version information.");
    Argh.AddLong("verbose", 'v').SetBool().SetDesc("More verbose.");
    Argh.StartParse(argc, argv);
    bool anything = false;
    verbose = 0;
    for(;;)
    {
        int c = Argh.GetParam();
        if(c == -1)break;
        
        switch(c)
        {
            case 'V': printf("%s\n", VERSION); return 0;
            case 'v': if(Argh.GetBool())++verbose;else --verbose;
                      break;
            case 'h': goto Usage;
            default:
            {
                typedef double valtype;
                string s = Argh.GetString();
                unsigned a=0, b=s.size();
                vector<valtype> keys, values;
                bool tmpset=false, k=false;
                for(a=0; a<b; ++a)
                {
                    const char *begin = s.c_str() + a;
                    char *endptr;
                    
                    errno = 0;
                    valtype tmp = strtod(begin, &endptr);
                    //valtype tmp = strtol(begin, &endptr, 10);
                    
                    if(errno == ERANGE)
                    {
                        /* FIXME: Handle this properly */
                        endptr = const_cast<char *> (begin);
                    }
                    
                    if(endptr == begin)
                    {
                        goto Got;
                    }
                    else
                    {
                        a += (endptr-begin)-1;
                        tmpset = true;
                    Got:
                        if(tmpset)
                            (k?values:keys).push_back(tmp);
                        tmpset = false;
                        if(a<b && s[a]=='=')k=!k;
                    }
                }
                
                algofinder<fraction<long long> > gruu;
                gruu.Guess(keys, values);
                
                anything = true;
            }
        }
    }
    if(!Argh.ok())return -1;
    if(!anything)
    {
Usage:
        printf(
            "algoguesser v"VERSION" - Copyright (C) 1992,2001 Bisqwit (http://iki.fi/bisqwit/)\n"
            "Guesses formula\n"
            "\nUsage: %s [<option> [<...>]] [<serie>=<serie> [<...>]]\n"
            "\nSerie format: <number>[,<number>[<...>]]\n"
            "\nOptions:\n", argv[0]);
        Argh.ListOptions();
        return 0;
    }
    
//  algofinder<int> gruu;
//  gruu.Test();
    
    return 0;
}
